# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

######################### We start with some black magic to print on failure.

# Change 1..1 below to 1..last_test_to_print .
# (It may become useful if the test is moved to ./t subdirectory.)

BEGIN { $| = 1; print "1..2\n"; }
END {print "not ok 1\n" unless $loaded;}


use XML::Node;

$loaded = 1;
print "ok 1\n";

######################### End of black magic.

# Insert your test code below (better if it prints "ok 13"
# (correspondingly "not ok 13") depending on the success of chunk 13
# of the test code):

my $suite_name;
my $testcase_name;

register(">TestSuite>SuiteName","char" => \$ suite_name);
register(">TestSuite>TestCase>Name","char" => \$testcase_name);
register(">TestSuite>TestCase","end" => \& handle_testcase_end);

parse("test.xml");

print "ok 2\n";

sub handle_testcase_end
{
    print "Test Case [$testcase_name] found in Test Suite [$suite_name]\n";
    $testcase_name = "";
}




